@@ -6,12 +6,9 @@
 #' eXtreme Gradient Boosting classification.
 #' Calls [xgboost::xgb.train()] from package \CRANpkg{xgboost}.
 #'
-#' If not specified otherwise, the evaluation metric is set to the default `"logloss"`
-#' for binary classification problems and set to `"mlogloss"` for multiclass problems.
-#' This was necessary to silence a deprecation warning.
-#'
-#' Note that using the `watchlist` parameter directly will lead to problems when wrapping this [mlr3::Learner] in a
-#' `mlr3pipelines` `GraphLearner` as the preprocessing steps will not be applied to the data in the watchlist.
+#' Note that using the `evals` parameter directly will lead to problems
+#' when wrapping this [mlr3::Learner] in a `mlr3pipelines` `GraphLearner`
+#' as the preprocessing steps will not be applied to the data in `evals`.
 #' See the section *Early Stopping and Validation* on how to do this.
 #'
 #' @template note_xgboost
@@ -29,13 +26,17 @@
 #'   - Actual default: 1.
 #'   - Adjusted default: 0.
 #'   - Reason for change: Reduce verbosity.
+#' - `verbosity`:
+#'   - Actual default: 1.
+#'   - Adjusted default: 0.
+#'   - Reason for change: Reduce verbosity.
 #'
 #' @section Early Stopping and Validation:
 #' In order to monitor the validation performance during the training, you can set the `$validate` field of the Learner.
 #' For information on how to configure the validation set, see the *Validation* section of [mlr3::Learner].
 #' This validation data can also be used for early stopping, which can be enabled by setting the `early_stopping_rounds` parameter.
 #' The final (or in the case of early stopping best) validation scores can be accessed via `$internal_valid_scores`, and the optimal `nrounds` via `$internal_tuned_values`.
-#' The internal validation measure can be set via the `eval_metric` parameter that can be a [mlr3::Measure], a function, or a character string for the internal xgboost measures.
+#' The internal validation measure can be set via the `custom_metric` parameter that can be a [mlr3::Measure], a function, or a character string for the internal xgboost measures.
 #' Using an [mlr3::Measure] is slower than the internal xgboost measures, but allows to use the same measure for tuning and validation.
 #'
 #' @inheritSection mlr_learners_regr.xgboost Offset
@@ -86,54 +87,58 @@ LearnerClassifXgboost = R6Class("LearnerClassifXgboost",
           if (is.null(param_vals$early_stopping_rounds)) {
             stop("Parameter 'early_stopping_rounds' must be set to use internal tuning.")
           }
-          if (is.null(param_vals$eval_metric)) {
-            stop("Parameter 'eval_metric' must be set explicitly when using internal tuning.")
+          if (is.null(param_vals$custom_metric) && is.null(param_vals$eval_metric)) {
+            stop("Parameter 'custom_metric' or 'eval_metric' must be set explicitly when using internal tuning.")
           }
           assert_integerish(domain$upper, len = 1L, any.missing = FALSE) }, .parent = topenv()),
-        disable_in_tune = list(early_stopping_rounds = NULL)
+        disable_in_tune = list(early_stopping_rounds = NULL),
+        init = 1000L
       )
 
       ps = ps(
         alpha                       = p_dbl(0, default = 0, tags = "train"),
         approxcontrib               = p_lgl(default = FALSE, tags = "predict"),
         base_score                  = p_dbl(default = 0.5, tags = "train"),
-        booster                     = p_fct(c("gbtree", "gblinear", "dart"), default = "gbtree", tags = c("train", "control")),
+        booster                     = p_fct(c("gbtree", "gblinear", "dart"), default = "gbtree", tags = "train"),
         callbacks                   = p_uty(default = list(), tags = "train"),
         colsample_bylevel           = p_dbl(0, 1, default = 1, tags = "train"),
         colsample_bynode            = p_dbl(0, 1, default = 1, tags = "train"),
-        colsample_bytree            = p_dbl(0, 1, default = 1, tags = c("train", "control")),
+        colsample_bytree            = p_dbl(0, 1, default = 1, tags = "train"),
         device                      = p_uty(default = "cpu", tags = "train"),
         disable_default_eval_metric = p_lgl(default = FALSE, tags = "train"),
         early_stopping_rounds       = p_int(1L, default = NULL, special_vals = list(NULL), tags = "train"),
-        eta                         = p_dbl(0, 1, default = 0.3, tags = c("train", "control")),
-        eval_metric                 = p_uty(tags = "train", custom_check = crate({function(x) check_true(any(is.character(x), is.function(x), test_multi_class(x, c("MeasureClassifSimple", "MeasureBinarySimple"))))})),
+        eta                         = p_dbl(0, 1, default = 0.3, tags = "train"),
+        evals                       = p_uty(default = NULL, tags = "train"),
+        eval_metric                 = p_uty(tags = "train"),
+        custom_metric               = p_uty(tags = "train", custom_check = crate({function(x) check_true(any(is.function(x), test_multi_class(x, c("MeasureClassifSimple", "MeasureBinarySimple"))))})),
+        extmem_single_page          = p_lgl(default = FALSE, tags = "train"),
         feature_selector            = p_fct(c("cyclic", "shuffle", "random", "greedy", "thrifty"), default = "cyclic", tags = "train", depends = quote(booster == "gblinear")),
-        gamma                       = p_dbl(0, default = 0, tags = c("train", "control")),
+        gamma                       = p_dbl(0, default = 0, tags = "train"),
         grow_policy                 = p_fct(c("depthwise", "lossguide"), default = "depthwise", tags = "train", depends = quote(tree_method == "hist")),
         interaction_constraints     = p_uty(tags = "train"),
         iterationrange              = p_uty(tags = "predict"),
         lambda                      = p_dbl(0, default = 1, tags = "train"),
-        lambda_bias                 = p_dbl(0, default = 0, tags = "train", depends = quote(booster == "gblinear")),
         max_bin                     = p_int(2L, default = 256L, tags = "train", depends = quote(tree_method == "hist")),
+        max_cached_hist_node        = p_int(default = 65536L, tags = "train", depends = quote(tree_method == "hist")),
+        max_cat_to_onehot           = p_int(tags = "train"),
+        max_cat_threshold           = p_dbl(tags = "train"),
         max_delta_step              = p_dbl(0, default = 0, tags = "train"),
-        max_depth                   = p_int(0L, default = 6L, tags = c("train", "control")),
+        max_depth                   = p_int(0L, default = 6L, tags = "train"),
         max_leaves                  = p_int(0L, default = 0L, tags = "train", depends = quote(grow_policy == "lossguide")),
         maximize                    = p_lgl(default = NULL, special_vals = list(NULL), tags = "train"),
-        min_child_weight            = p_dbl(0, default = 1, tags = c("train", "control")),
-        missing                     = p_dbl(default = NA, tags = c("train", "predict"), special_vals = list(NA, NA_real_, NULL)),
-        monotone_constraints        = p_uty(default = 0, tags = c("train", "control"), custom_check = crate(function(x) { checkmate::check_integerish(x, lower = -1, upper = 1, any.missing = FALSE) })), # nolint
+        min_child_weight            = p_dbl(0, default = 1, tags = "train"),
+        missing                     = p_dbl(default = NA, tags = "predict", special_vals = list(NA, NA_real_, NULL)),
+        monotone_constraints        = p_uty(default = 0, tags = "train", custom_check = crate(function(x) { checkmate::check_integerish(x, lower = -1, upper = 1, any.missing = FALSE) })), # nolint
         nrounds                     = p_nrounds,
         normalize_type              = p_fct(c("tree", "forest"), default = "tree", tags = "train", depends = quote(booster == "dart")),
-        nthread                     = p_int(1L, default = 1L, tags = c("train", "control", "threads")),
-        ntreelimit                  = p_int(1L, default = NULL, special_vals = list(NULL), tags = "predict"),
-        num_parallel_tree           = p_int(1L, default = 1L, tags = c("train", "control")),
-        objective                   = p_uty(default = "binary:logistic", tags = c("train", "predict", "control")),
+        nthread                     = p_int(1L, init = 1L, tags = c("train", "threads")),
+        num_parallel_tree           = p_int(1L, default = 1L, tags = "train"),
+        objective                   = p_uty(default = "binary:logistic", tags = c("train", "predict")),
         one_drop                    = p_lgl(default = FALSE, tags = "train", depends = quote(booster == "dart")),
         print_every_n               = p_int(1L, default = 1L, tags = "train", depends = quote(verbose == 1L)),
-        process_type                = p_fct(c("default", "update"), default = "default", tags = "train"),
         rate_drop                   = p_dbl(0, 1, default = 0, tags = "train", depends = quote(booster == "dart")),
         refresh_leaf                = p_lgl(default = TRUE, tags = "train"),
-        reshape                     = p_lgl(default = FALSE, tags = "predict"),
+        seed                        = p_int(tags = "train"),
         seed_per_iteration          = p_lgl(default = FALSE, tags = "train"),
         sampling_method             = p_fct(c("uniform", "gradient_based"), default = "uniform", tags = "train", depends = quote(booster == "gbtree")),
         sample_type                 = p_fct(c("uniform", "weighted"), default = "uniform", tags = "train", depends = quote(booster == "dart")),
@@ -141,21 +146,19 @@ LearnerClassifXgboost = R6Class("LearnerClassifXgboost",
         save_period                 = p_int(0, default = NULL, special_vals = list(NULL), tags = "train"),
         scale_pos_weight            = p_dbl(default = 1, tags = "train"),
         skip_drop                   = p_dbl(0, 1, default = 0, tags = "train", depends = quote(booster == "dart")),
-        strict_shape                = p_lgl(default = FALSE, tags = "predict"),
-        subsample                   = p_dbl(0, 1, default = 1, tags = c("train", "control")),
+        subsample                   = p_dbl(0, 1, default = 1, tags = "train"),
         top_k                       = p_int(0, default = 0, tags = "train", depends = quote(feature_selector %in% c("greedy", "thrifty") && booster == "gblinear")),
         training                    = p_lgl(default = FALSE, tags = "predict"),
         tree_method                 = p_fct(c("auto", "exact", "approx", "hist", "gpu_hist"), default = "auto", tags = "train", depends = quote(booster %in% c("gbtree", "dart"))),
         tweedie_variance_power      = p_dbl(1, 2, default = 1.5, tags = "train", depends = quote(objective == "reg:tweedie")),
         updater                     = p_uty(tags = "train"), # Default depends on the selected booster
-        verbose                     = p_int(0L, 2L, default = 1L, tags = "train"),
-        watchlist                   = p_uty(default = NULL, tags = "train"),
+        use_rmm                     = p_lgl(tags = "train"),
+        validate_features           = p_lgl(default = TRUE, tags = "predict"),
+        verbose                     = p_int(0L, 2L, init = 0L, tags = "train"),
+        verbosity                   = p_int(0L, 2L, init = 0L, tags = "train"),
         xgb_model                   = p_uty(default = NULL, tags = "train")
       )
 
-      # custom defaults
-      ps$set_values(nrounds = 1000L, nthread = 1L, verbose = 0L)
-
       super$initialize(
         id = "classif.xgboost",
         predict_types = c("response", "prob"),
@@ -207,6 +210,21 @@ LearnerClassifXgboost = R6Class("LearnerClassifXgboost",
         private$.validate = assert_validate(rhs)
       }
       private$.validate
+    },
+
+    #' @field model (any)\cr
+    #' The fitted model. Only available after `$train()` has been called.
+    model = function(rhs) {
+      if (!missing(rhs)) {
+        if (inherits(rhs, "xgb.Booster")) {
+          rhs = list(
+            structure("wrapper", model = rhs)
+          )
+        }
+        self$state$model = rhs
+      }
+      # workaround https://github.com/Rdatatable/data.table/issues/7456
+      attributes(self$state$model[[1]])$model
     }
   ),
   private = list(
@@ -230,20 +248,7 @@ LearnerClassifXgboost = R6Class("LearnerClassifXgboost",
         stopf("objective = 'multi:softmax' does not work with predict_type = 'prob'")
       }
 
-      switch(pv$objective,
-        "multi:softprob" =,
-        "multi:softmax" = {
-          # add the number of classes 'num_class'
-          pv$num_class = nlvls
-
-          # we have to set this to avoid a deprecation warning
-          if (is.null(pv$feval)) pv$eval_metric = pv$eval_metric %??% "mlogloss"
-        },
-
-        "binary:logistic" = {
-          if (is.null(pv$feval)) pv$eval_metric = pv$eval_metric %??% "logloss"
-        }
-      )
+      if (pv$objective %in% c("multi:softmax", "multi:softprob")) pv$num_class = nlvls
 
       data = task$data(cols = task$feature_names)
       # recode to 0:1 so that for the binary case the positive class translates to 1 (#32)
@@ -305,41 +310,57 @@ LearnerClassifXgboost = R6Class("LearnerClassifXgboost",
           xgboost::setinfo(xgb_valid_data, "base_margin", base_margin)
         }
 
-        pv$watchlist = c(pv$watchlist, list(test = xgb_valid_data))
+        pv$evals = c(pv$evals, list(test = xgb_valid_data))
       }
 
       # set internal validation measure
-      if (inherits(pv$eval_metric, "Measure")) {
+      if (inherits(pv$custom_metric, "Measure")) {
         n_classes = length(task$class_names)
-        measure = pv$eval_metric
+        measure = pv$custom_metric
 
         fun = if (pv$objective == "binary:logistic" && measure$predict_type == "prob" && inherits(measure, "MeasureBinarySimple")) {
-            xgboost_binary_binary_prob
-          } else if (pv$objective == "binary:logistic" && measure$predict_type == "prob" && inherits(measure, "MeasureClassifSimple")) {
-            xgboost_binary_classif_prob
-          } else if (pv$objective == "binary:logistic" && measure$predict_type == "response") {
-            xgboost_binary_response
-          } else if (pv$objective == "multi:softprob" && measure$predict_type == "prob") {
-            xgboost_multiclass_prob
-          } else if (pv$objective %in% c("multi:softmax", "multi:softprob") && measure$predict_type == "response") {
-            xgboost_multiclass_response
-          } else {
-            stop("Only 'binary:logistic', 'multi:softprob' and 'multi:softmax' objectives are supported.")
-          }
-
-        pv$eval_metric =  mlr3misc::crate({function(pred, dtrain) {
-            scores = fun(pred, dtrain, measure, n_classes)
-             list(metric = measure$id, value = scores)
-          }}, n_classes, measure, fun)
+          xgboost_binary_binary_prob
+        } else if (pv$objective == "binary:logistic" && measure$predict_type == "prob" && inherits(measure, "MeasureClassifSimple")) {
+          xgboost_binary_classif_prob
+        } else if (pv$objective == "binary:logistic" && measure$predict_type == "response") {
+          xgboost_binary_response
+        } else if (pv$objective == "multi:softprob" && measure$predict_type == "prob") {
+          xgboost_multiclass_prob
+        } else if (pv$objective %in% c("multi:softmax", "multi:softprob") && measure$predict_type == "response") {
+          xgboost_multiclass_response
+        } else {
+          stop("Only 'binary:logistic', 'multi:softprob' and 'multi:softmax' objectives are supported.")
+        }
 
+        pv$custom_metric =  mlr3misc::crate({function(pred, dtrain) {
+          scores = fun(pred, dtrain, measure, n_classes)
+          list(metric = measure$id, value = scores)
+        }}, n_classes, measure, fun)
         pv$maximize = !measure$minimize
       }
 
-      invoke(xgboost::xgb.train, data = xgb_data, .args = pv)
+      model = xgboost::xgb.train(
+        params = pv[names(pv) %in% formalArgs(xgboost::xgb.params)],
+        data = xgb_data,
+        nrounds = pv$nrounds,
+        evals = pv$evals,
+        custom_metric = pv$custom_metric,
+        verbose = pv$verbose,
+        print_every_n = pv$print_every_n,
+        early_stopping_rounds = pv$early_stopping_rounds,
+        maximize = pv$maximize,
+        save_period = pv$save_period,
+        save_name = pv$save_name,
+        callbacks = pv$callbacks %??% list()
+      )
+
+      # workaround https://github.com/Rdatatable/data.table/issues/7456
+      list(
+        structure("wrapper", model = model)
+      )
     },
 
     .predict = function(task) {
-
       pv = self$param_set$get_values(tags = "predict")
       model = self$model
       response = prob = NULL
@@ -352,10 +373,9 @@ LearnerClassifXgboost = R6Class("LearnerClassifXgboost",
 
       newdata = as_numeric_matrix(ordered_features(task, self))
       pred = invoke(predict, model, newdata = newdata, .args = pv)
-
       if (nlvls == 2L) { # binaryclass
         if (pv$objective == "multi:softprob") {
-          prob = matrix(pred, ncol = nlvls, byrow = TRUE)
+          prob = matrix(pred, ncol = nlvls, byrow = FALSE)
           colnames(prob) = lvls
         } else {
           prob = pvec2mat(pred, lvls)
@@ -364,7 +384,7 @@ LearnerClassifXgboost = R6Class("LearnerClassifXgboost",
         if (pv$objective == "multi:softmax") {
           response = lvls[pred + 1L]
         } else {
-          prob = matrix(pred, ncol = nlvls, byrow = TRUE)
+          prob = pred
           colnames(prob) = lvls
         }
       }
@@ -381,38 +401,62 @@ LearnerClassifXgboost = R6Class("LearnerClassifXgboost",
 
     .hotstart = function(task) {
       model = self$model
-      pars = self$param_set$get_values(tags = "train")
-      pars_train = self$state$param_vals
-      if (!is.null(pars_train$early_stopping_rounds)) {
+      pv = self$param_set$get_values(tags = "train")
+      pv_train = self$state$param_vals
+      if (!is.null(pv_train$early_stopping_rounds)) {
         stopf("The parameter `early_stopping_rounds` is set. Early stopping and hotstarting are incompatible.")
       }
 
       # Calculate additional boosting iterations
       # niter in model and nrounds in ps should be equal after train and continue
-      pars$nrounds = pars$nrounds - pars_train$nrounds
+      nrounds = pv$nrounds - pv_train$nrounds
 
       # Construct data
       nlvls = length(task$class_names)
       data = task$data(cols = task$feature_names)
       label = nlvls - as.integer(task$truth())
-      data = xgboost::xgb.DMatrix(data = as_numeric_matrix(data), label = label)
+      xgb_data = xgboost::xgb.DMatrix(data = as_numeric_matrix(data), label = label)
 
-      invoke(xgboost::xgb.train, data = data, xgb_model = model, .args = pars)
+      if (nrounds > 0) {
+        model = xgboost::xgb.train(
+          params = pv[names(pv) %in% formalArgs(xgboost::xgb.params)],
+          data = xgb_data,
+          nrounds = nrounds,
+          evals = pv$evals,
+          custom_metric = pv$custom_metric,
+          verbose = pv$verbose,
+          print_every_n = pv$print_every_n,
+          early_stopping_rounds = pv$early_stopping_rounds,
+          maximize = pv$maximize,
+          save_period = pv$save_period,
+          save_name = pv$save_name,
+          xgb_model = model,
+          callbacks = pv$callbacks %??% list()
+        )
+      }
+
+      list(
+        structure("wrapper", model = model)
+      )
     },
 
     .extract_internal_tuned_values = function() {
       if (is.null(self$state$param_vals$early_stopping_rounds)) {
         return(NULL)
       }
-      list(nrounds = self$model$best_iteration)
+      list(nrounds = attributes(self$model)$early_stop$best_iteration)
     },
 
     .extract_internal_valid_scores = function() {
-      if (is.null(self$model$evaluation_log)) {
-        NULL
+      if (is.null(attributes(self$model)$evaluation_log)) {
+        return(named_list())
       }
-      iter = if (!is.null(self$model$best_iteration)) self$model$best_iteration else self$model$niter
-      as.list(self$model$evaluation_log[
+      iter = attributes(self$model)$early_stop$best_iteration
+      if (is.null(iter)) {
+        iter = xgboost::xgb.get.num.boosted.rounds(self$model)
+      }
+      log = attributes(self$model)$evaluation_log
+      as.list(log[
         iter,
         set_names(get(".SD"), gsub("^test_", "", colnames(get(".SD")))),
         .SDcols = patterns("^test_")
@@ -468,11 +512,9 @@ xgboost_multiclass_prob = function(pred, dtrain, measure, n_classes, ...) {
   # label is a vector of labels (0, 1, ..., n_classes - 1)
   truth = factor(xgboost::getinfo(dtrain, "label"), levels = seq_len(n_classes) - 1L)
 
-  # pred is a vector of log odds for each class
-  # matrix must be filled by row
-  pred_mat = matrix(pred, ncol = n_classes, byrow = TRUE)
+  # pred is a matrix of log odds for each class
   # transform log odds to probabilities
-  pred_exp = exp(pred_mat)
+  pred_exp = exp(pred)
   pred_mat = pred_exp / rowSums(pred_exp)
   colnames(pred_mat) = levels(truth)
 
@@ -483,11 +525,8 @@ xgboost_multiclass_response = function(pred, dtrain, measure, n_classes, ...) {
   # label is a vector of labels (0, 1, ..., n_classes - 1)
   truth = factor(xgboost::getinfo(dtrain, "label"), levels = seq_len(n_classes) - 1L)
 
-  # pred is a vector of log odds for each class
-  # matrix must be filled by row
-  pred_mat = matrix(pred, ncol = n_classes, byrow = TRUE)
-
-  response = factor(max.col(pred_mat, ties.method = "random") - 1, levels = levels(truth))
+  # pred is a matrix of log odds for each class
+  response = factor(max.col(pred, ties.method = "random") - 1, levels = levels(truth))
   measure$fun(truth, response)
 }
 
