@@ -35,25 +35,18 @@ convert_ratio = function(pv, target, ratio, n) {
   )
 }
 
-
-
-
-ranger_selected_features = function(self) {
-  if (is.null(self$model)) {
+ranger_selected_features = function(model, feature_names) {
+  if (is.null(model)) {
     stopf("No model stored")
   }
 
-  splitvars = ranger::treeInfo(object = self$model, tree = 1)$splitvarName
+  splitvars = ranger::treeInfo(object = model, tree = 1)$splitvarName
   i = 2
-  while (i <= self$model$num.trees &&
-      !all(self$state$feature_names %in% splitvars)) {
-    sv = ranger::treeInfo(object = self$model, tree = i)$splitvarName
+  while (i <= model$num.trees && !all(feature_names %in% splitvars)) {
+    sv = ranger::treeInfo(object = model, tree = i)$splitvarName
     splitvars = union(splitvars, sv)
     i = i + 1
   }
 
-  # order the names of the selected features in the same order as in the task
-  self$state$feature_names[self$state$feature_names %in% splitvars]
+  splitvars[!is.na(splitvars)]
 }
-
-
